<?php
if (!defined('__TYPECHO_ROOT_DIR__')) exit;
/**
 * 导航菜单
 * 
 * @package TeMenu 
 * @author 绛木子
 * @version 1.0.0
 * @link http://lixianhua.com
 */
class TeMenu_Plugin implements Typecho_Plugin_Interface
{
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     * 
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function activate()
    {
        // 增加导航菜单管理
		Helper::addPanel(3, 'TeMenu/panel.php', '菜单', '管理菜单', 'administrator');
        // 增加action
		Helper::addAction('TeMenu','TeMenu_Action');
    }
    
    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     * 
     * @static
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function deactivate(){
        Helper::removeAction('TeMenu');
        Helper::removePanel(3,'TeMenu/panel.php');
    }
    
    /**
     * 获取插件配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form 配置面板
     * @return void
     */
    public static function config(Typecho_Widget_Helper_Form $form)
    {
        // 存储导航菜单数据
		$json = '{"header":{"menu":[{"target":"0","icon":"","type":"custom","name":"\u9996\u9875","id":"{siteUrl}"}],"title":"\u5bfc\u822a\u83dc\u5355"}}';
		$json = str_replace('{siteUrl}', Helper::options()->siteUrl,$json);
		$navigation = new Typecho_Widget_Helper_Form_Element_Textarea('navigation', NULL, $json, _t('菜单数据'), _t('菜单数据，请通过菜单管理工具进行编辑'));
		$form->addInput($navigation);
    }
    
    /**
     * 个人用户的配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form
     * @return void
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form){}
    
    /**
	  * 显示菜单
	  * @param string $slug 菜单名称
	  * @param string $options 菜单参数
	  * 
	  * item:菜单元素
	  * itemTag:菜单元素的标签
	  * childTag:子菜单标签
	  * childClass:子菜单css类
	  * caret:是否显示子菜单标记
	  * caretDiv:子菜单标记代码
	  */
	public static function show($slug,$options=''){
		$navigation = json_decode(Typecho_Widget::widget('Widget_Options')->plugin('TeMenu')->navigation,true);
		if(!isset($navigation[$slug])){
			return '';
		}
		$menu = $navigation[$slug]['menu'];
		$options = new Typecho_Config($options);
		$options->setDefault(array(
			'wrapTag'=>'ul',
			'wrapClass'=>'',
			'itemTag'=>'li',
			'itemClass'=>'',
			'item'=>'<a {current} href="{url}" {target}>{icon} {name} {caret}</a>',
			'childTag'=>'ul',
			'childClass'=>'child',
			'caret'=>' <i class="fa fa-caret-down"></i>',
		));
		$html = '';
		if($options->wrapTag){
			$html = '<' . $options->wrapTag . (empty($options->wrapClass)
                    ? '' : ' class="' . $options->wrapClass . '"') . '>';
		}
		
		$html .= self::parseMenu($menu,$options);
		if($options->wrapTag){
			$html .= '</' . $options->wrapTag . '>';
		}
		echo $html;
	}
	// 解析菜单数据
	private static function parseMenu($menu,&$options){
		$html = '';
		$archive = Typecho_Widget::widget('Widget_Archive');
		foreach($menu as $k=>$v){
			if($v['type']=='category'){
				$category = self::getCategory($v['id']);
				$v['url'] = $category['permalink'];
				$v['slug'] = $category['slug'];
				$v['current'] = $archive->is('category',$category['slug']);
			}elseif($v['type']=='page'){
				$page = self::getPage($v['id']);
				$v['url'] = $page['permalink'];
				$v['slug'] = $page['slug'];
				$v['current'] = $archive->is('page',$page['slug']);
			}else{
				$url = $archive->request->getRequestUrl();
				$v['url'] = $v['id'];
				$v['current'] = $url === $v['url'];
			}
			
			if(isset($v['children'])){
				$v['caret'] = $options->caret;
			}else{
				$v['caret'] = '';
			}
			$v['target'] = isset($v['target']) && $v['target'] ? 'target="_blank"' : '';

			$v['icon'] = isset($v['icon']) ? '<i class="'.$v['icon'].'"></i>' : '';
			$v['current'] && $v['current'] = 'class="current"';
			
			$html .= $options->itemTag ? '<'. $options->itemTag . (empty($options->itemClass)
							? '' : ' class="' . $options->itemClass . '"') . '>' : '';
			
			$html .= str_replace(array('{url}','{name}','{current}','{icon}','{caret}','{target}'),
				array($v['url'],$v['name'],$v['current'],$v['icon'],$v['caret'],$v['target']),$options->item);
			
			if(isset($v['children'])){
				if($options->childTag){
					$html .= '<' . $options->childTag . (empty($options->childClass)
							? '' : ' class="' . $options->childClass . '"') . '>';
				}
				
				$html .= self::parseMenu($v['children'],$options);
				
				$html .= $options->childTag ? '</'. $options->childTag . '>' : '';
			}
			$html .= $options->itemTag ? '</'. $options->itemTag . '>' : '';
		}
		return $html;
	}

    // 获取单个页面
	public static function getPage($cid){
		return Typecho_Widget::widget('Widget_Page_Query')->getPage($cid);
	}
	// 获取分类
	public static function getCategory($mid){
		return Typecho_Widget::widget('Widget_Metas_Category_List')->getCategory($mid);
	}

    // 后台显示菜单操作列表
	public static function showNavigationDrag($navigation){
		$html = '';
		if(empty($navigation)) return '';
		
		$html .='<ol class="dd-list">';
		foreach($navigation as $k=>$v){
			$v['target'] = isset($v['target']) && $v['target'] ? 1 : 0;
			$v['icon'] = isset($v['icon']) ? $v['icon'] : '';

			$html .= '<li class="dd-item" data-id="'.$v['id'].'" data-name="'.$v['name'].'" data-type="'.$v['type'].'" data-icon="'.$v['icon'].'" data-target="'.$v['target'].'">';
			$html .= '<div class="dd-handle">Drag</div><div class="dd-content">'.$v['name'].'</div><div class="dd-setting">Setting</div><div class="dd-remove">Remove</div>';
			$html .= '<div class="dd-setdiv"><label>名称</label><input class="w-100 text dd-name" type="text" value="'.$v['name'].'" />';
			if($v['type']=='custom'){
				$html .= '<label>链接</label><input class="w-100 text dd-url" type="text" value="'.$v['id'].'" />';
			}
			$html .= '<label>新窗口打开</label><input class="text dd-target" type="checkbox" value="1" '. ($v['target'] ? 'checked' : '') .'/>用新窗口打开链接';
			$html .= '<label>图标样式</label><input class="w-100 text dd-icon" type="text" value="'.$v['icon'].'"/>';
			$html .= '</div>';
			if(isset($v['children'])){
				$html .= self::showNavigationDrag($v['children']);
			}
			$html .= '</li>';
		}
		$html .= '</ol>';
		return $html;
	}
}
class Widget_Page_Query extends Widget_Abstract_Contents
{
	 /**
     * 所有页面哈希表 
     * 
     * @var array
     * @access private
     */
    private $_map = array();
    /**
     * 执行函数
     *
     * @access public
     * @return void
     */
    public function execute()
    {
        $select = $this->select()->where('table.contents.type = ?', 'page')
        ->where('table.contents.created < ?', $this->options->gmtTime)
        ->order('table.contents.order', Typecho_Db::SORT_ASC);

        //去掉自定义首页
        $frontPage = explode(':', $this->options->frontPage);
        if (2 == count($frontPage) && 'page' == $frontPage[0]) {
            $select->where('table.contents.cid <> ?', $frontPage[1]);
        }

        $rs = $this->db->fetchAll($select, array($this, 'push'));
		foreach($rs as $v){
			$this->_map[$v['cid']] = $v;
		}
    }
	/**
     * 获取单个页面
     * 
     * @param integer $cid 
     * @access public
     * @return mixed
     */
    public function getPage($cid)
    {
        return isset($this->_map[$cid]) ? $this->_map[$cid] : NULL;
    }
}